#ifndef COMPLEX_NUMBER_H
#define COMPLEX_NUMBER_H

/**
 *Represents complex numbers.
 *A complex number has a real part and an imaginary part.
 *
 *For example z = 3 + 2i
 */
template <typename _Tp> class complex {
 private:
  /** The Real part */
  _Tp realPart;
  /** The imaginary part*/
  _Tp img;
 public:
  /**
   *The default constructor
   */
  complex();
  /**
   *The copy constructor
   *Creates an identical copy of an other complex number
   *@param cp is the object to copy
   */
  complex(const complex<_Tp> & cp);
  /**
   *Creates a complex number with it's real and imaginary
   *parts defined.
   *@param realNum is the value of the real part
   *@part imgNum is the value of the imaginary part
   */
  complex(const _Tp realNum, const _Tp imgNum);
  /**
   *Provides access to the real part of the complex number
   *@return the value of the real part of the number
   */
  const _Tp getReal() const;
  /**
   *Provides access to the imaginary part of the complex number
   *@return the value of the imaginary part of the number
   */
  const _Tp getImg() const;
  /**
   *The assignement operator (=)
   *The imaginary part will be equal to zero
   *@param real is the value of the real part
   *@return itself
   */
  complex<_Tp>& operator =(const _Tp real);
  /**
   *The assignement operator (=)
   *The object will be the identical copy of the parameter
   @param cp is the object which is copied
   *@return itself
   */
  complex<_Tp>& operator =(const complex<_Tp> & cp);
  /**
   *The assignement addition operator(+=)
   *The value of the real part of the object will be added to the parameter
   *@param addVal is the value to add to the real part of the number
   *@return itself
   */
  complex<_Tp>& operator +=(const _Tp addVal);
  /**
   *The assignement addition operator(+=)
   *The real and imaginary values of the number will be added to the
   *parameter
   *@param addNum is complex number to add to this number
   *@return itself
   */
  complex<_Tp>& operator +=(const complex<_Tp> & addNum);

  /*The two *= variants*/
  complex<_Tp>& operator *=(const _Tp mulNum);
  complex<_Tp>& operator *=(const complex<_Tp>& z);
};
/**
 *The addition operator(+)
 *A new object will be created where the real part will be
 *equal to the real part of this object plus the value of the parameter
 *@param addVal is the value to add to the real part of the number
 *@param z is the complex number
 *@return a new object where the real part will be equal to the real part
 *value of this object plus the parameter
 */
template <typename _Tp> complex<_Tp> operator +(const _Tp addVal,
												const complex<_Tp> & z);

/**
 *The addition operator(+)
 *A new object will be created where the real part will be
 *equal to the real part of this object plus the value of the parameter
 *@param z is the complex number
 *@param addVal is the value to add to the real part of the number
 *@return a new object where the real part will be equal to the real part
 *value of this object plus the parameter
 */
template <typename _Tp> complex<_Tp> operator +(const complex<_Tp>& z,
												const _Tp addVal);

/**
 *The addition operator(+)
 *A new object will be created where the real and imaginary part will be
 *equal to the real and imaginary parts of the two complex numbers.
 *@param z1 is the first complex number
 *@param z2 is the second complex number
 *@return a new complex number which is the addition of the parameters
 */
template <typename _Tp> complex<_Tp> operator +(const complex<_Tp>& z1,
												const complex<_Tp>& z2);
//The tree * operatros
//cases are
//z * num
//num * z
//z * z
template <typename _Tp> complex<_Tp> operator *(const _Tp num,
												const complex<_Tp>& z);
template <typename _Tp> complex<_Tp> operator *(const complex<_Tp>& z,
												const _Tp num);
template <typename _Tp> complex<_Tp> operator *(const complex<_Tp>& z1,
												const complex<_Tp>& z2);

typedef complex<int> icomplex;
typedef complex<double> dcomplex;
typedef complex<float> fcomplex;


template<typename _Tp> complex<_Tp>::complex() : realPart(_Tp()), img(_Tp()) {}

template<typename _Tp> complex<_Tp>::complex(const complex<_Tp>& cp)
  : realPart(cp.realPart), img(cp.img) {}

template<typename _Tp> complex<_Tp>::complex(const _Tp realNum,
											 const _Tp imgNum)
  : realPart(realNum), img(imgNum) {}


template<typename _Tp> const _Tp complex<_Tp>::getReal() const {
  return realPart;
}


template<typename _Tp> const _Tp complex<_Tp>::getImg() const { return img; }


template<typename _Tp> complex<_Tp>& complex<_Tp>::operator =(const _Tp real) {
  realPart = real;
  return *this;
}

template<typename _Tp> complex<_Tp>& complex<_Tp>::operator
=(const complex<_Tp>& cp) {
  realPart = cp.realPart;
  img = cp.img;
  return *this;
}

template<typename _Tp> complex<_Tp>& complex<_Tp>::operator
+=(const _Tp addVal) {
  realPart += addVal;
  return *this;
}

template<typename _Tp> complex<_Tp>& complex<_Tp>::operator
+=(const complex<_Tp>& addNum) {
  realPart += addNum.realPart;
  img += addNum.img;
  return *this;
}

template<typename _Tp> complex<_Tp> operator
+(const _Tp addVal, const complex<_Tp>& z) {
  complex<_Tp> cpx = z;
  cpx += addVal;
  return cpx;
}

template<typename _Tp> complex<_Tp> operator
+(const complex<_Tp>& z, const _Tp addVal) {
  complex<_Tp> cpx = z;
  cpx += addVal;
  return cpx;
}

template<typename _Tp> complex<_Tp> operator
+(const complex<_Tp>& z1, const complex<_Tp>& z2) {
  complex<_Tp> cpx = z1;
  cpx += z2;
  return cpx;
}


template<typename _Tp> complex<_Tp>& complex<_Tp>::operator
*=(const _Tp mulNum) {
  realPart *= mulNum;
  img *= mulNum;
  return *this;
}


template<typename _Tp> complex<_Tp>& complex<_Tp>::operator
*=(const complex<_Tp>& z) {
  /*
	this = 2 + 3i
	z   = 1 + i
	this *=z => this = this * z
	this = (2 + 3i) * (1 + i)
	2 * 1 + 2 * i + 3 * 1 * i + 3*i^2
	2 + 2i + 3i -3
   */
  _Tp r1 = realPart * z.realPart;
  _Tp r2 = img * z.img;
  _Tp i1 = realPart * z.img;
  _Tp i2 = img * z.realPart;
  realPart = r1 - r2;
  img = i1 + i2;
  return *this;
}


template<typename _Tp> complex<_Tp> operator
*(const _Tp num, const complex<_Tp>& z) {
  complex<_Tp> ret = z;
  ret *= num;
  return ret;
}


template<typename _Tp> complex<_Tp> operator
*(const complex<_Tp>& z, const _Tp num) {
  complex<_Tp> ret = z;
  ret *= num;
  return ret;
}


template<typename _Tp> complex<_Tp> operator
*(const complex<_Tp>& z1, const complex<_Tp>& z2) {
  complex<_Tp> ret = z1;
  ret *= z2;
  return ret;
}

#endif
